#ifndef __MODULE_API_H_
#define __MODULE_API_H_

#ifdef __cplusplus
extern "C" {
#endif
#include "common_defs.h"
//包头
#define SYNCWORD 0xEFAA


#define MSG_LENGTH_SIZE      0x02
#define MSG_CHECKSUM_SIZE    0x01

#define USER_NAME_SIZE       32
#define MAX_USER_COUNTS      50
#define VERSION_INFO_BUFFER_SIZE 32


/* msg face direction */
#define FACE_DIRECTION_UP   	 0x10       // face up
#define FACE_DIRECTION_DOWN      0x08       // face down
#define FACE_DIRECTION_LEFT      0x04       // face left
#define FACE_DIRECTION_RIGHT     0x02       // face right
#define FACE_DIRECTION_MIDDLE    0x01       // face middle


typedef uint8_t s_face_dir;
typedef uint8_t MSGID;

typedef enum _SYS_STATE
{
    MS_STANDBY = 0,  // module is IDLE, waiting for commands
    MS_BUSY    = 1,  // module is working for processing commands
    MS_ERROR   = 2,  // module is in error state. can't work properly
    MS_INVALID = 3,  // module is not initialized
}SYS_STATE;

typedef enum _NOTE_TYPE
{
    NID_READY         = 0,  // module ready for handling request (command)
    NID_FACE_STATE    = 1,  // the detected face status description
    NID_UNKNOWNERROR  = 2,  // unknown error
    NID_OTA_DONE      = 3,  // OTA upgrading processing done
    NID_EYE_STATE     = 4,  // the face in the eye_close status
    NID_AUTHORIZATION = 8,  // license authorization failed
}NOTE_TYPE;

typedef enum _MSG_TYPE
{
    MSGTYPE_REPLY = 0, //REPLY
    MSGTYPE_NOTE  = 1, //NOTE
    MSGTYPE_IMAGE = 2, //IMAGE
    MSGTYPE_MAX
}MSG_TYPE;

typedef enum _MSG_ID
{
    MID_REPLY=0x00,
    MID_NOTE=0x01,

    MID_RESET=0x10,
    MID_GETSTATUS=0x11,
    MID_VERIFY=0x12,
    MID_ENROLL=0x13,
    MID_ENROLL_SINGLE=0x1D,

    MID_DELUSER=0x20,
    MID_DELALL=0x21,       //timeout: 10 seconds
    MID_GETUSERINFO=0x22,
    MID_FACERESET=0x23,
    MID_GET_ALL_USERID=0x24,

    MID_ENROLL_ITG=0x26,

    MID_GET_VERSION=0x30,

    MID_INIT_ENCRYPTION=0x50,
    MID_SET_RELEASE_ENC_KEY=0x52,
    MID_SET_DEBUG_ENC_KEY=0x53,

    MID_POWERDOWN=0xED,

    MID_DEMOMODE = 0xFE,

    MID_MAX = 0xFF,
}MSG_ID;

typedef enum _MSG_ERRCODE
{
    MR_SUCCESS    = 0,     // success
    MR_REJECTED   = 1,     // module rejected this command
    MR_ABORTED    = 2,     // algo aborted
    MR_FAILED4_CAMERA = 4, // camera open failed
    MR_FAILED4_UNKNOWNREASON = 5, // UNKNOWN_ERROR
    MR_FAILED4_INVALIDPARAM = 6,  // invalid param
    MR_FAILED4_NOMEMORY = 7,      // no enough memory
    MR_FAILED4_UNKNOWNUSER = 8,   // no user enrolled
    MR_FAILED4_MAXUSER = 9,       // exceed maximum user number
    MR_FAILED4_FACEENROLLED = 10, // this face has been enrolled
    MR_FAILED4_LIVENESSCHECK = 12,// liveness check failed
    MR_FAILED4_TIMEOUT = 13,      // exceed the time limit
    MR_FAILED4_AUTHORIZATION = 14,// authorization failed
    MR_FAILED4_CAMERAFOV = 15,    // camera fov test failed
    MR_FAILED4_CAMERAQUA = 16,    // camera quality test failed
    MR_FAILED4_CAMERASTRU = 17,   // camera structure test failed
    MR_FAILED4_BOOT_TIMEOUT = 18, // boot up timeout
    MR_FAILED4_READ_FILE = 19,    // read file failed
    MR_FAILED4_WRITE_FILE = 20,   // write file failed
    MR_FAILED4_NO_ENCRYPT = 21,   // encrypt must be set
}MSG_ERRCODE;

typedef enum _FACE_STATE
{
    FACE_STATE_NORMAL     = 0, 	// normal state, the face is available
    FACE_STATE_NOFACE     = 1, 	// noface detected
    FACE_STATE_TOOUP      = 2,  // face is too up side
    FACE_STATE_TOODOWN    = 3,  // face is too down side
    FACE_STATE_TOOLEFT    = 4,  // face is too left side
    FACE_STATE_TOORIGHT   = 5,  // face is too right side
    FACE_STATE_TOOFAR     = 6,  // face is too far
    FACE_STATE_TOOCLOSE   = 7,  // face is too near
    FACE_STATE_EYEBROW_OCCLUSION = 8,   // eyebrow occlusion
    FACE_STATE_EYE_OCCLUSION     = 9,   // eye occlusion
    FACE_STATE_FACE_OCCLUSION    = 10,  // face occlusion
    FACE_STATE_DIRECTION_ERROR   = 11,  // face direction error
    FACE_STATE_EYE_CLOSE_STATUS_OPEN_EYE = 12,  // eye close time out
    FACE_STATE_EYE_CLOSE_STATUS          = 13,  // confirm eye close status
    FACE_STATE_EYE_CLOSE_UNKNOW_STATUS   = 14,  // eye close unknow status
}FACE_STATE;




typedef struct {
    uint8_t mid;
    uint8_t size[2];
    uint8_t data[0];
} s_msg;

typedef struct {
    uint8_t mid;
    uint8_t result;
    uint8_t data[0];
} s_msg_reply_data;

typedef struct {
    uint8_t nid;
    uint8_t data[0];
} s_msg_note_data;



//MID_INIT_ENCRYPTION data
typedef struct {
    uint8_t seed[4];    // random data as a seed
    uint8_t mode;       // reserved for selecting encrytion algorithm
    uint8_t crttime[4];
} s_msg_init_encryption_data;

//verify
typedef struct {
    uint8_t pd_rightaway; // power down right away after verifying
    uint8_t timeout;      // timeout, unit second, default 10s
} s_msg_verify_data;

//enroll
typedef struct {
    uint8_t admin;                           //the user will be set to admin
    uint8_t user_name[USER_NAME_SIZE];
    s_face_dir face_direction;               //交互录入时需要，单帧录入忽略
    uint8_t timeout;                         //timeout 1-255, unit second default 10s
} s_msg_enroll_data;

typedef struct {
    uint8_t admin;                           //the user will be set to admin
    uint8_t user_name[USER_NAME_SIZE];
    s_face_dir face_direction;               //多角度交互录入时需要，单帧录入忽略
    uint8_t enroll_type;                     //0:多角度交互录入           1:单帧录入
    uint8_t enable_duplicate;                //0:disable   1:enable
    uint8_t timeout;                         //timeout 1-255, unit second default 10s
    uint8_t reserved[3];
} s_msg_enroll_itg;

typedef struct {
    uint8_t user_id_heb;
    uint8_t user_id_leb;
    uint8_t face_direction;
} s_msg_reply_enroll_data;

typedef struct
{
    uint8_t version_info[VERSION_INFO_BUFFER_SIZE];
}s_msg_reply_version_data;

/* message reply data definitions */
typedef struct {
    uint8_t user_id_heb;
    uint8_t user_id_leb;
    uint8_t user_name[USER_NAME_SIZE];
    uint8_t admin;
    uint8_t unlockStatus;
} s_msg_reply_verify_data;


typedef struct {
    int16_t state; // corresponding to FACE_STATE_*

    // position
    int16_t left;  // in pixel
    int16_t top;
    int16_t right;
    int16_t bottom;

    // pose
    int16_t yaw;   // up and down in vertical orientation
    int16_t pitch; // right or left turned in horizontal orientation
    int16_t roll;  // slope
} s_note_data_face;

typedef struct {
    uint8_t user_id_heb;
    uint8_t user_id_leb;
    uint8_t user_name[USER_NAME_SIZE];
    uint8_t admin;
} s_msg_reply_getuserinfo_data;

typedef struct {
    uint8_t user_counts;                         // number of enrolled users
    uint8_t users_id[MAX_USER_COUNTS * 2];  //use 2 bytes to save a user ID and save high eight bits firstly
} s_msg_reply_all_userid_data;

int module_init(const char* devName);
//得到模组状态
int module_getStatus();
//重置模组
int module_reset();
//获取注册全部用户信息
int module_getAllUserId(s_msg_reply_all_userid_data *data);
//获取注册某个用户信息
int module_getUserInfo(s_msg_reply_getuserinfo_data *data);
//删除用户
int module_deleteUser(const uint16_t id);
//删除全部用户
int module_deleteAll();
//单帧注册
int module_enrollSingle(const s_msg_enroll_data data);
//识别
int module_verify(s_msg_reply_verify_data *data);
//获取版本信息
int module_getVersion(s_msg_reply_version_data *version);

#ifdef __cplusplus
}
#endif
#endif
